#!/usr/bin/perl

use strict;
use warnings;
use utf8;


## CONST VARIABLE
##
my @_ACTION	= ();
my @_TARGET	= ();
my $_DEBUG	= 0;
my $_USER	= "API User";
my $_PASS	= "API Password";
my $_FQDN	= "Zabbix Server FQDN";
my $_API	= "https://".$_FQDN."/api_jsonrpc.php";


## DEBUG MESSAGE
##   TRUE::  Stdout debug message
##   FALSE:: N/A
sub getDebugPrint{
	my $MESSAGE	= "$_[0]";

	# Get script debug message.
	if($_DEBUG){
		print "[".time."]:: ".$MESSAGE."\n";
	}
}


## USER AUTHENTICATION
##   TRUE::  Authenticate token
##   FALSE:: Error code '-1'
sub getAuthToken{
	my %RESULT;
	my $RETURN	= "-1";
	my $API		= "$_[0]";
	my $USER	= "$_[1]";
	my $PASS	= "$_[2]";
	my $HEADER	= "Content-Type: application/json-rpc";
	my $QUERY	= '{"jsonrpc":"2.0", "method":"user.login", "params":{"user":"'.$USER.'", "password":"'.$PASS.'"}, "id":1, "auth":null}';

	# Get API authenticate token.
	&getDebugPrint("curl -s -X POST -H \"$HEADER\" -d \'$QUERY\' -k $API");
	foreach my $_RESP(split(/,/, `curl -s -X POST -H "$HEADER" -d \'$QUERY\' -k $API`)){
		if($_RESP =~ /\"(.*)\":(.*)/){
			my $KEY   =  $1;
			my $VALUE =  $2;
			$VALUE    =~ s/(^\")|(^\{)|(\}$)//;
			$VALUE    =~ s/(\".*)//;
			$RESULT{$KEY} = $VALUE;
			&getDebugPrint($KEY.": ".$VALUE);
		}
	}
	foreach my $_KEY(sort keys(%RESULT)){
		if($_KEY eq "result"){
			$RETURN = "$RESULT{$_KEY}";
			&getDebugPrint("token: ".$RESULT{$_KEY});
		}
	}
	&getDebugPrint("---");
	return "$RETURN";
}


## USER LOGOUT
##   TRUE::  Logout session response
##   FALSE:: Error code '-1'
sub setAuthLogout{
	my $RETURN	= "-1";
	my $API		= "$_[0]";
	my $TOKEN	= "$_[1]";
	my $HEADER	= "Content-Type: application/json-rpc";
	my $QUERY	= '{"jsonrpc":"2.0", "method":"user.logout", "params":[], "auth":"'.$TOKEN.'", "id":1}';

	# API user logout
	&getDebugPrint("curl -s -X POST -H \"$HEADER\" -d \'$QUERY\' -k $API");
	foreach my $_RESP(`curl -s -X POST -H "$HEADER" -d \'$QUERY\' -k $API`){
		&getDebugPrint("response: ".$_RESP);
		$RETURN = 0;
	}
	&getDebugPrint("---");
	return "$RETURN";
}


## GET TRIGGER ACTION LIST
##   TRUE::  Count of trigger action
##   FALSE:: Error Code '-1'
sub getTriggerActionList{
	my @RESULT;
	my $RETURN	= "-1";
	my $API		= "$_[0]";
	my $TOKEN	= "$_[1]";
	my $HEADER	= "Content-Type: application/json-rpc";
	my $FILTER	= '{"eventsource":0}';
	my $QUERY	= '{"jsonrpc":"2.0", "method":"action.get", "params":{"output":"extend", "filter":'.$FILTER.'}, "auth":"'.$TOKEN.'", "id":1}';

	# Get trigger action parameter.
	&getDebugPrint("curl -s -X POST -H \"$HEADER\" -d \'$QUERY'\ -k $API");
	foreach my $_RESP(split(/\:\[\{|\}\,\{|\}\]\,/, `curl -s -X POST -H "$HEADER" -d \'$QUERY'\ -k $API`)){
		if($_RESP =~ /eventsource/){
			my %JSON;
			foreach my $_OBJECT(split(/,/, $_RESP)){
				if($_OBJECT =~ /\"(.*)\":\"(.*)\"/){
					my $KEY     = $1;
					my $VALUE   = $2;
					$JSON{$KEY} = $VALUE;
					&getDebugPrint($KEY.": ".$VALUE);
				}
			}
			push(@RESULT, \%JSON);
			&getDebugPrint("...");
		}
	}
	if(scalar(@RESULT)>0){
		@_ACTION = @RESULT;
		$RETURN  = scalar(@RESULT);
	}
	&getDebugPrint("---");
	return "$RETURN";
}


## SEARCH TRIGGER ACTION ID
##   TRUE::  Count of trigger action
##   FALSE:: Error code '-1'
sub getTriggerActionID{
	my @RESULT;
	my $RETURN	= "-1";
	my $NAME	= "$_[0]";

	# Search trigger name.
	&getDebugPrint("name: ".$NAME);
	foreach my $_LINE(@_ACTION){
		my $ID   = "";
		my %JSON = %$_LINE;
		foreach my $_KEY(sort keys(%JSON)){

			# Write action ID.
			if($_KEY eq "actionid"){ $ID = $JSON{$_KEY};     }

			# For all action ID.
			if($NAME eq "all"){ push(@RESULT, $ID); last;    }

			# For search action name.
			if($JSON{$_KEY} eq "$NAME"){ push(@RESULT, $ID); }
		}
		&getDebugPrint("actionid: ".$ID);
	}
	if(scalar(@RESULT)>0){
		@_TARGET = @RESULT;
		$RETURN  = scalar(@RESULT);
	}
	&getDebugPrint("---");
	return "$RETURN";
}


## SWITCH TRIGGER ACTION FLAG
##   TRUE::  Count of flag changes.
##   FALSE:: Error code '-1'
sub setTriggerActionFlag{
	my $RETURN	= "-1";
	my $API		= "$_[0]";
	my $TOKEN	= "$_[1]";
	my $FLAG	= "$_[2]" eq "enable" ? 0 : 1;
	my $HEADER	= "Content-Type: application/json-rpc";
	my $QUERY_H	= '{"jsonrpc":"2.0", "method":"action.update", "params":{"actionid":';
	my $QUERY_F	= ', "status":'.$FLAG.'}, "auth":"'.$TOKEN.'", "id":1}';

	# Switching action flag.
	foreach my $_LINE(@_TARGET){
		my $QUERY = $QUERY_H.$_LINE.$QUERY_F;
		&getDebugPrint("curl -s -X POST -H \"$HEADER\" -d \'$QUERY'\ -k $API");
		foreach my $_RESP(`curl -s -X POST -H "$HEADER" -d \'$QUERY\' -k $API`){
			&getDebugPrint("response: ".$_RESP);
			$RETURN++;
		}
	}
	if($RETURN>-1){
		$RETURN++;
	}
	&getDebugPrint("---");
	return "$RETURN";
}


## MAIN ROUTINE
##
##   > 1st arg, [Default: disable]
##   >   Enable/Disable switching process
##   > 2nd arg, [Default: all]
##   >   Target trigger action name
##
##   - Enable all trigger.
##     $ api_zabbix_failover.pl enable
##     $ api_zabbix_failover.pl enable all
##
##   - Enable specifying the trigger name.
##     $ api_zabbix_failover.pl enable "Emergency Level-0 [CLEAR]"
##
##   - Disable all trigger.
##     $ api_zabbix_failover.pl disable
##     $ api_zabbix_failover.pl disable all
##
##   - disable specifying the trigger name.
##     $ api_zabbix_failover.pl disable "Emergency Level-0 [CLEAR]"
##
my $_SWITCH;
my $_FILTER;
if(defined($ARGV[0])){ $_SWITCH = $ARGV[0]; }else{ $_SWITCH = "disable"; }
if(defined($ARGV[1])){ $_FILTER = $ARGV[1]; }else{ $_FILTER = "all";     }
&getDebugPrint("SWITCH: ".$_SWITCH);
&getDebugPrint("FILTER: ".$_FILTER);

# Operation of zabbix action trigger.
my $_TOKENS = &getAuthToken($_API, $_USER, $_PASS);
if($_TOKENS eq "-1"){
	print "Incorrect user name or password or account is temporarily blocked.\n";
	exit 1;
}
if(&getTriggerActionList($_API, $_TOKENS) eq "-1"){;
	print "Session terminated, re-login, please.\n";
	exit 1;
}
if(&getTriggerActionID($_FILTER) eq "-1"){
	print "Incorrect query syntax is blocked.\n";
	exit 1;
}
if(&setTriggerActionFlag($_API, $_TOKENS, $_SWITCH) eq "-1"){
	print "Status change query execute error.\n";
	exit 1;
}
if(&setAuthLogout($_API, $_TOKENS) eq "-1"){
	print "ZabbixZPI user logout process failed.\n";
	exit 1;
}

exit 0;
